#!/usr/bin/env python3
"""
HTTP server for metrics
"""

import traceback
from urllib.parse import urlparse
from urllib import parse
from http.server import BaseHTTPRequestHandler
from http.server import HTTPServer
from . import collector


class EsSearchExporterHandler(BaseHTTPRequestHandler):
    def __init__(self, config, kerberos, tls, *args, **kwargs):
        self._config = config
        self._kerberos = kerberos
        self._tls = tls
        BaseHTTPRequestHandler.__init__(self, *args, **kwargs)

    def do_GET(self):
        config = self._config
        url = urlparse(self.path)
        if url.path == "/esdata":
            params = parse.parse_qs(parse.unquote(self.path.split("?", 1)[1]))
            if "search" not in params:
                self.send_response(400)
                self.end_headers()
                msg = "Missing 'search' from parameters %s" % params["searcdh"]
                self.wfile.write(msg.encode())
                return
            output_data = b""
            for search in params["search"]:
                if search not in config["searches"]:
                    self.send_response(400)
                    self.end_headers()
                    self.wfile.write(
                        "Search {} not found in config".format(search).encode()
                    )
                    return
                try:
                    output = collector.search_es(
                        config["searches"][search], self._kerberos, self._tls
                    )
                    output_data = output_data + output

                except:
                    self.send_response(500)
                    self.end_headers()
                    self.wfile.write(traceback.format_exc().encode())

            self.send_response(200)
            self.end_headers()
            self.wfile.write(output_data)

        elif url.path == "/":
            self.send_response(200)
            self.end_headers()
            self.wfile.write(
                b"""<html>
      <head><title>Elasticsearch Data Exporter</title></head>
      <body>
      <h1>Elasticsearch Data Exporter</h1>
      <p>Visit <code>/metrics?search=example</code> to use.</p>
      </body>
      </html>"""
            )
        else:
            self.send_response(404)
            self.end_headers()


def start_http_server(config, port, kerberos, tls):
    GetHandler = lambda *args, **kwargs: EsSearchExporterHandler(
        config, kerberos, tls, *args, **kwargs
    )
    server = HTTPServer(("", port), GetHandler)
    print("Starting server, use <Ctrl-C> to stop\n", config)
    server.serve_forever()



import json
import requests
from requests_kerberos import HTTPKerberosAuth
import time
from pandas.io.json import json_normalize
from datetime import datetime


def search_es(config, kerberos, tls):
    pro_rows = "\n"
    host = config["es_host"]
    try:
        a = None
        if kerberos:
            a = HTTPKerberosAuth()
        s = ""
        if tls:
            s = "s"
        r = requests.post(
            "http{}://{}/{}/_search".format(s, host, config["es_index"]),
            headers={"Content-Type": "application/json"},
            data=json.dumps(config["es_query"]),
            auth=a,
            timeout=10,
        )
    except:
        raise Exception(
            "Cannot connect to Elasticsearch host: {}\n{}".format(
                host, "http{}://{}/{}/_search".format(s, host, config["es_index"])
            )
        )
    if r.status_code == 200:
        es_data = r.json()["hits"]["hits"]
        es_list = []
        for x in es_data:
            es_list.append(x.get("_source"))
        df = json_normalize(es_list)
        for _, rows in df.iterrows():
            rows.dropna(axis=0, how="any", inplace=True)
            df_keys = rows.keys()
            row = rows.to_dict()
            stats_tags = ""
            for t in config["prometheus_tags"]:
                try:
                    stats_tags += ',{}="{}"'.format(t.replace(".", "_"), row[t])
                except KeyError:
                    pass
            stats_tags = "{" + stats_tags[1:] + "}"
            timestamp = int(
                datetime.strptime(
                    row[config["timestamp_tag"]], "%Y-%m-%dT%H:%M:%S.%f%z"
                ).timestamp()
                * 1000
            )
            for k in df_keys:
                if isinstance(row[k], int) or isinstance(row[k], float):
                    pro_row = "{}{} {} {}\n".format(
                        k.replace(".", "_"), stats_tags, row[k], timestamp
                    )
                    pro_rows = pro_rows + pro_row
                else:
                    pass
    else:
        raise Exception("Query failed: {}".format(r.json()))

    return pro_rows.encode()



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="es_data_exporter",
    version="0.1.4",
    author="kasen",
    author_email="kasen@outlook.com",
    description=("Elasticsearch data exporter to the Prometheus ."),
    long_description=(
        "See https://github.com/yaule/es_data_exporter/blob/master/README.md for documentation."
    ),
    license="MIT",
    keywords="prometheus exporter network monitoring elastic search",
    url="https://github.com/yaule/es_data_exporter",
    scripts=["scripts/es_data_exporter"],
    packages=["es_data_exporter"],
    install_requires=[
        "pyyaml>=3.13",
        "requests>=2.21.0",
        "requests-kerberos>=0.12.0",
        "pandas>=0.23.4",
    ],
    classifiers=[
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
    ],
)
